/*
 * Servlet.java
 *
 * Created on March 6, 2002, 9:57 AM
 */

package org.semanticweb.triple.application;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.semanticweb.daml.*;
import org.semanticweb.dl.*;


public class Servlet extends HttpServlet {

  // CONFIGURE HERE
  final static String xsbProgram = "/U/sintek/bin/xsb";
  final static String xsbMemory = "16000"; // kb
  final static String racerProgram = "/U/sintek/bin/racer";
  final static String dotProgram = "/U/sintek/lopt/graphviz/bin/dot";
  final static String dotFonts = "/U/sintek/etc";
  final static String logDir = "/U/sintek/ljava/tomcat/logs/triple/";
  // END CONFIGURE

  public void doPost(HttpServletRequest request, 
	HttpServletResponse response) throws IOException, ServletException {
    run(request, response);
  }

  public void doGet(HttpServletRequest request,
        HttpServletResponse response) throws IOException, ServletException {
    run(request, response);
  }

  void run(HttpServletRequest request,
        HttpServletResponse response) throws IOException, ServletException {

    String tripleProgram = request.getParameter("program");
    String damlOntology = request.getParameter("daml");
    if (tripleProgram != null)
      runTRIPLE(tripleProgram, request, response);
    else if (damlOntology != null)
      runDAML(damlOntology, request, response);
    else {
      response.setContentType("text/html");
      PrintWriter htmlWriter = response.getWriter();
      htmlWriter.println("<html>");
      htmlWriter.println("<body text=#000000 bgcolor=#DCDCDC>");
      htmlWriter.println("Error: illegal use of servlet");
      htmlWriter.println("</body>");
      htmlWriter.println("</html>");
    }
  }

  void runTRIPLE(String tripleProgram, HttpServletRequest request,
        HttpServletResponse response) throws IOException, ServletException {

    String timestamp = Long.toString(new Date().getTime());
    String tmpTripleFile = logDir + "triple" + timestamp + ".triple";
    String tmpXSBFile = logDir + "triple" + timestamp + ".P";

    writeFile(tmpTripleFile, tripleProgram);

    // create html result page
    response.setContentType("text/html");
    PrintWriter htmlWriter = response.getWriter();
    htmlWriter.println("<html>");
    htmlWriter.println("<body text=#000000 bgcolor=#DCDCDC>");
    htmlWriter.println("<h2>Output From TRIPLE Engine</h2>");
    htmlWriter.println("<pre>");
    try {
      Commandline.compileTripleProgram(tmpTripleFile, tmpXSBFile, 
        logDir, timestamp, true); // true -> servlet mode
      String[] xsbCommand = new String[] { xsbProgram, "--noprompt", "--quiet",
        "-m", xsbMemory, "-e", "['" + tmpXSBFile + "']." };
      runExternalProgram(xsbCommand, htmlWriter, "halt.");
    } catch (Exception e) {
      htmlWriter.println("Error: " + e);
    }
    htmlWriter.println("</pre>");
    htmlWriter.println("<hr>");
    htmlWriter.println("<h2>TRIPLE Program</h2>");
    htmlWriter.println("<pre>");
    htmlWriter.println(tripleProgram);
    htmlWriter.println("</pre>");
    htmlWriter.println("</body>");
    htmlWriter.println("</html>");
  }

  void runDAML(String damlOntology, HttpServletRequest request,
        HttpServletResponse response) throws IOException, ServletException {

    String timestamp = Long.toString(new Date().getTime());
    String tmpDamlFile = logDir + "daml" + timestamp + ".daml";
    String tmpRaceriFileName = "daml-ri" + timestamp + ".xml";
    String tmpRaceriFile = logDir + tmpRaceriFileName;
    String tmpRaceroFile = logDir + "daml-ro" + timestamp + ".xml";
    String tmpDotFile = logDir + "daml" + timestamp + ".dot";
    String tmpPSFileName = "daml" + timestamp + ".ps";
    String tmpPSFile = logDir + tmpPSFileName;
    String tmpGifFileName = "daml" + timestamp + ".gif";
    String tmpGifFile = logDir + tmpGifFileName;

    writeFile(tmpDamlFile, damlOntology);

    // create html result page
    response.setContentType("text/html");
    PrintWriter htmlWriter = response.getWriter();
    htmlWriter.println("<html>");
    htmlWriter.println("<body text=#000000 bgcolor=#DCDCDC>");
    htmlWriter.println("<h2>Output From DAML+OIL Servlet</h2>");
    try {
      Reader in = new FileReader(tmpDamlFile);
      String namespace = "http://www.examples.org#";
      DamlModel model = new DamlModel(in, namespace);
      KB kb = model.toDL();
      KB shiq = kb.toSHIQ();
      PrintWriter out = new PrintWriter(new FileWriter(tmpRaceriFile));
      Map abbrevs = shiq.printXML(out);
      HashMap namespaces = new HashMap();
      for (Iterator iter = abbrevs.entrySet().iterator(); iter.hasNext();) {
        Map.Entry entry = (Map.Entry)iter.next();
        namespaces.put(entry.getValue(), entry.getKey());
      }
      Racer racer = 
        new Racer(racerProgram, tmpRaceriFile, namespace, namespaces, tmpRaceroFile);
      racer.runRacer();
      KB racerOutputKB = racer.getTaxonomyKB();
      // output in Lisp syntax
      /* works, but who wants to see this :-)
      Syntax syntax = new Syntax(racerOutputKB);
      htmlWriter.println("<pre>");
      racerOutputKB.print(syntax, htmlWriter);
      htmlWriter.println("</pre>");
      */
      // output as dot file
      Syntax syntax = new DotSyntax(racerOutputKB);
      PrintWriter out2 = new PrintWriter(new FileWriter(tmpDotFile));
      racerOutputKB.print(syntax, out2);
      // process with dot to create PS and gif files
      Runtime.getRuntime().exec(
        new String[] { dotProgram, "-Tps", tmpDotFile, "-o", tmpPSFile } )
        .waitFor();
      Runtime.getRuntime().exec(
        new String[] { dotProgram, "-Tgif", tmpDotFile, "-o", tmpGifFile },
        new String[] { "DOTFONTPATH=" + dotFonts } )
        .waitFor();
      htmlWriter.println("<img alt=\"taxonomy\" src=\"files/" + tmpGifFileName + "\">");
      htmlWriter.println("<br><br>");
      htmlWriter.println("<a href=\"files/" + tmpPSFileName + "\">Postscript file</a>");
      htmlWriter.println("&nbsp;");
      htmlWriter.println("<a href=\"files/" + tmpRaceriFileName + "\">RACER input file</a>");
      htmlWriter.println("&nbsp;");
      htmlWriter.println("<a href=\"http://triple.semanticweb.org\">triple.semanticweb.org</a>");
    } catch (Exception e) {
      htmlWriter.println("Error: " + e);
    } catch (com.hp.hpl.mesa.rdf.jena.model.RDFError re) {
      htmlWriter.println("RDF error: " + re);
    }

    htmlWriter.println("</body>");
    htmlWriter.println("</html>");
  }


  // auxiliaries

  void writeFile(String fileName, String contents) {
    try {
      PrintWriter writer = new PrintWriter(new FileWriter(fileName));
      if (contents != null)
	writer.println(contents);
      writer.flush();
      writer.close();
    } catch (Exception e) { System.err.println(e); }
  }

  void runExternalProgram(String[] command, PrintWriter writer, 
      String sendString) throws Exception {
    Process process = Runtime.getRuntime().exec(command);
    PrintWriter outputStream = new PrintWriter(process.getOutputStream(), true);
    BufferedReader reader = 
      new BufferedReader(new InputStreamReader(process.getInputStream()));
    if (sendString != null)
      outputStream.println(sendString);
    outputStream.close(); // ??
    String line = reader.readLine();
    while (line != null) {
      writer.println(line);
      line = reader.readLine();
    }
  }

}

