package org.semanticweb.triple.inference;

import java.io.*;
import java.util.*;

/**
 *  Description of the Class
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public class SymbolhandlerHash implements Symbolhandler {
    HashSymbolTable fSymbols = null;
    HashSymbolTable pSymbols = null;
    HashSymbolTable strings = null;


    /**
     *  Constructor for the SymbolhandlerHash object
     */
    public SymbolhandlerHash() {
        fSymbols = new HashSymbolTable();
        pSymbols = new HashSymbolTable();
        strings = new HashSymbolTable();
    }


    /**
     *  Gets the contentFunctionSymbols attribute of the SymbolhandlerHash
     *  object
     *
     *@param  i  Description of the Parameter
     *@return    The contentFunctionSymbols value
     */
    public String getContentFunctionSymbols(int i) {
        SymbolWrapper sw = fSymbols.getContent(i);
        if (sw != null) {
            return sw.s;
        }
        else {
            return null;
        }
    }


    /**
     *  Gets the contentPredicateSymbols attribute of the SymbolhandlerHash
     *  object
     *
     *@param  i  Description of the Parameter
     *@return    The contentPredicateSymbols value
     */
    public String getContentPredicateSymbols(int i) {
        SymbolWrapper sw = pSymbols.getContent(i);
        if (sw != null) {
            return sw.s;
        }
        else {
            return null;
        }
    }


    /**
     *  Gets the contentStrings attribute of the SymbolhandlerHash object
     *
     *@param  i  Description of the Parameter
     *@return    The contentStrings value
     */
    public String getContentStrings(int i) {
        SymbolWrapper sw = (SymbolWrapper) strings.getContent(i);
        if (sw != null) {
            return sw.s;
        }
        return null;
    }


    /**
     *  Gets the insertMode attribute of the SymbolhandlerHash object
     *
     *@return    The insertMode value
     */
    public boolean getInsertMode() {
        return fSymbols.getInsertMode();
    }


    /**
     *  Gets the sizeFunctionSymbols attribute of the SymbolhandlerHash object
     *
     *@return    The sizeFunctionSymbols value
     */
    public int getSizeFunctionSymbols() {
        return fSymbols.getSize();
    }


    /**
     *  Gets the sizePredicateSymbols attribute of the SymbolhandlerHash object
     *
     *@return    The sizePredicateSymbols value
     */
    public int getSizePredicateSymbols() {
        return pSymbols.getSize();
    }


    /**
     *  Gets the sizeStrings attribute of the SymbolhandlerHash object
     *
     *@return    The sizeStrings value
     */
    public int getSizeStrings() {
        return strings.getSize();
    }


    /**
     *  Description of the Method
     */
    public void rollback() {
        fSymbols.rollback();
        pSymbols.rollback();
        strings.rollback();
    }


    /**
     *  Description of the Method
     *
     *@param  s  Description of the Parameter
     *@param  a  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int searchAndInsertFunctionSymbols(String s, int a) {
        return fSymbols.searchAndInsert(new SymbolWrapper(s, a));
    }


    /**
     *  Description of the Method
     *
     *@param  s  Description of the Parameter
     *@param  a  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int searchAndInsertPredicateSymbols(String s, int a) {
        int i = pSymbols.searchAndInsert(new SymbolWrapper(s, a));
        //System.out.println("Symbol: " + s + "/" + i);
        return i;
    }


    /**
     *  Description of the Method
     *
     *@param  s  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int searchAndInsertStrings(String s) {
        SymbolWrapper sw = new SymbolWrapper(s);
        return strings.searchAndInsert(sw);
    }


    /**
     *  Description of the Method
     *
     *@param  s  Description of the Parameter
     *@param  a  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int searchFunctionSymbols(String s, int a) {
        return fSymbols.search(new SymbolWrapper(s, a));
    }


    /**
     *  Description of the Method
     *
     *@param  s  Description of the Parameter
     *@param  a  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int searchPredicateSymbols(String s, int a) {
        int i = pSymbols.search(new SymbolWrapper(s, a));
        //System.out.println("Symbol: " + s + "/" + i);
        return i;
    }


    /**
     *  Description of the Method
     *
     *@param  s  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int searchStrings(String s) {
        SymbolWrapper sw = new SymbolWrapper(s);
        return strings.search(sw);
    }


    /**
     *  Sets the insertMode attribute of the SymbolhandlerHash object
     *
     *@param  insertMode  The new insertMode value
     */
    public void setInsertMode(boolean insertMode) {
        fSymbols.setInsertMode(insertMode);
        pSymbols.setInsertMode(insertMode);
        strings.setInsertMode(insertMode);
    }


    /**
     *  Sets the rollbackPoint attribute of the SymbolhandlerHash object
     */
    public void setRollbackPoint() {
        fSymbols.setRollback();
        pSymbols.setRollback();
        strings.setRollback();
    }

}
