package org.semanticweb.triple.parser;

import java.util.*;


public class LStruct extends LTerm {

  LTerm functor;
  ArrayList args; // of LTerm

  public LStruct() {
    args = new ArrayList();
  }

  public LStruct(LTerm functor, ArrayList args) {
    this.functor = functor;
    this.args = args;
  }

  public LStruct(String functor, ArrayList args) {
    this.functor = new LSymbol(functor);
    this.args = args;
  }

  public LStruct(LTerm functor, LTerm arg) {
    this.functor = functor;
    args = new ArrayList();
    args.add(arg);
  }

  public LStruct(String functor, LTerm arg) {
    this.functor = new LSymbol(functor);
    args = new ArrayList();
    args.add(arg);
  }

  public LStruct(String functor, LTerm arg1, LTerm arg2) {
    this.functor = new LSymbol(functor);
    args = new ArrayList();
    args.add(arg1);
    args.add(arg2);
  }

  public void putFunctor(LTerm functor) {
    this.functor = functor;
  }

  public void putArgs(LTerm term) {
    args.add(term);
  }

  public LTerm getFunctor() {
    return functor;
  }

  public ArrayList getArgs() {
    return args;
  }
  
  public LTerm normalize(HashMap abbreviations) {
    // if this is a FORALL or EXISTS, introduce new abbreviations
    // (in a cloned version!)
    if (functor instanceof LStruct) {
      LStruct funct = (LStruct)functor;
      LTerm functor2 = funct.getFunctor();
      if (functor2 instanceof LSymbol) {
        String functor2name = ((LSymbol)functor2).getSymbol();
        if (   functor2name.equalsIgnoreCase("FORALL")
            || functor2name.equalsIgnoreCase("EXISTS")) {
          ArrayList variables = funct.getArgs();
          abbreviations = (HashMap)abbreviations.clone();
          Context.addVariables(variables, abbreviations);
        }
      }
    }
    // do the normalization
    boolean changed = false;
    LTerm fexpansion = functor.normalize(abbreviations);
    if (functor != fexpansion) changed = true;
    ArrayList aexpansion = new ArrayList();
    if (args != null) {
      for (Iterator argIterator = args.iterator(); argIterator.hasNext();) {
	LTerm arg = (LTerm)argIterator.next();
        LTerm aexp = arg.normalize(abbreviations);
        if (arg != aexp) changed = true;
        aexpansion.add(aexp);
      }
    }
    if (changed)
      return new LStruct(fexpansion, aexpansion);
    else
      return this;
  }
  
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    if (functor.toString().equals("CLAUSE")) { // HACK
      buffer.append("\n");
    }
    buffer.append(functor.toString());
    if (args != null && args.size() > 0) {
      buffer.append("(");
      for (Iterator argIterator = args.iterator(); argIterator.hasNext();) {
	LTerm arg = (LTerm)argIterator.next();
	buffer.append(arg.toString());
	if (argIterator.hasNext())
	  buffer.append(", ");
      }
      buffer.append(")");
    }
    return buffer.toString();
  }

}

